using System;
using System.Text;
using System.Collections;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace Demo_2_Marshalling
{
	public class Window
	{
		const int MAXTITLE = 255;

		private static ArrayList mTitlesList;

		private delegate bool EnumDelegate(IntPtr hWnd, int lParam);

		[DllImport("user32.dll", EntryPoint="EnumDesktopWindows",
				ExactSpelling=false, CharSet=CharSet.Auto, SetLastError=true)]
		private static extern bool _EnumDesktopWindows(IntPtr hDesktop, 
			EnumDelegate lpEnumCallbackFunction, IntPtr lParam);

		[DllImport("user32.dll", EntryPoint="GetWindowText", 
				ExactSpelling=false, CharSet=CharSet.Auto, SetLastError=true)]
		private static extern int _GetWindowText(IntPtr hWnd,
			StringBuilder lpWindowText, int nMaxCount);

		private static bool EnumWindowsProc(IntPtr hWnd, int lParam)
		{
			string title = GetWindowText(hWnd);
			if (title.Length > 0)
			{
				mTitlesList.Add(title);
			}

			return true;
		}

		/// <summary>
		///       HWND .
		/// </summary>
		public static string GetWindowText(IntPtr hWnd)
		{
			StringBuilder title = new StringBuilder(MAXTITLE);
			int titleLength = _GetWindowText(hWnd, title, title.Capacity + 1);
			title.Length = titleLength;

			return title.ToString();
		}

		/// <summary>
		///      .
		/// </summary>
		public static string[] GetDesktopWindowsCaptions()
		{
			mTitlesList = new ArrayList();
			EnumDelegate enumfunc = new EnumDelegate(EnumWindowsProc);
			IntPtr hDesktop = IntPtr.Zero; //   
			bool success = _EnumDesktopWindows(hDesktop, enumfunc, IntPtr.Zero);

			if (success)
			{
				//      
				string[] titles = new string[mTitlesList.Count];
				mTitlesList.CopyTo(titles);
				return titles;
			}
			else
			{
				//     
				int errorCode = Marshal.GetLastWin32Error();

				string errorMessage = String.Format(
					"EnumDesktopWindows failed with code {0}.", errorCode);
				throw new Exception(errorMessage);
			}
		}
	}
}
